import React, { Component, } from 'react';
import { ScrollView, View, StyleSheet, } from 'react-native';
import Select from '../Select';
const NOOP = () => { };
export class Props {
    constructor() {
        this.mask = false;
        this.onChange = NOOP;
        this.onPress = NOOP;
    }
}
const styles = StyleSheet.create({
    mask: {
        position: 'absolute',
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
        justifyContent: 'space-between',
    },
    scroll: {
        opacity: 0,
        position: 'absolute',
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
    },
});
export default class Picker extends Component {
    constructor() {
        super(...arguments);
        this.containerHeight = 0;
        this.paddingVertical = 0;
        this.itemHeightList = [];
        this.itemHeight = 0;
        this.initCount = 0;
        this.scrollPoint = { x: 0, y: 0 };
        this.scrollTarget = null;
        this.scrollDraging = false;
        this.scrollMomentum = false;
        this.scrollFinish = false;
        this.scrollFake = null;
        this.scrollTimer = null;
        this.momentumTimer = null;
        this.onScroll = ({ nativeEvent }) => {
            this.scrollPoint = nativeEvent.contentOffset;
            if (this.scrollFake) {
                this.scrollFake.scrollTo(Object.assign({}, this.scrollPoint, { animated: false }));
            }
        };
        this.onScrollEnd = () => {
            this.scrollFinish = true;
            const { onChange, children } = this.props;
            const { y } = this.scrollPoint;
            let index = Math.round(y / this.itemHeight);
            if (this.scrollTarget) {
                this.scrollTarget.scrollTo(Object.assign({}, this.scrollPoint, { y: this.itemHeight * index, animated: true }));
            }
            if (index >= children.length)
                index = children.length - 1;
            if (index < 0)
                index = 0;
            onChange(children[index].props.value);
        };
        this.onScrollBeginDrag = () => {
            this.scrollFinish = false;
            this.scrollMomentum = false;
            this.scrollDraging = true;
            this.clearScrollTimer();
            this.clearMomentumTimer();
        };
        this.onScrollEndDrag = () => {
            this.scrollDraging = false;
            this.setScrollTimer();
        };
        this.onMomentumScrollBegin = () => {
            this.scrollMomentum = true;
            this.clearScrollTimer();
        };
        this.onMomentumScrollEnd = () => {
            this.scrollMomentum = false;
            this.setMomentumTimer();
        };
        this.onScrollReady = (scroll) => {
            this.scrollTarget = scroll;
        };
        this.onFakeReady = (scroll) => {
            this.scrollFake = scroll;
        };
        this.setScrollTimer = () => {
            this.clearScrollTimer();
            this.scrollTimer = setTimeout(() => {
                if (!this.scrollMomentum) {
                    this.onScrollEnd();
                }
            }, 16);
        };
        this.setMomentumTimer = () => {
            this.clearMomentumTimer();
            this.momentumTimer = setTimeout(() => {
                if (!this.scrollFinish) {
                    this.onScrollEnd();
                }
            }, 16);
        };
        this.getContainerStyle = () => (this.containerHeight ? {
            height: this.containerHeight,
        } : {});
        this.getSelectStyle = () => (this.paddingVertical ? {
            paddingVertical: this.paddingVertical,
        } : {});
        this.getItemStyle = (style) => {
            let style$ = style;
            if (!Array.isArray(style$)) {
                style$ = [style$];
            }
            style$ = [
                {
                    alignItems: 'center',
                    justifyContent: 'center',
                    height: 36,
                },
                ...style$,
            ];
            return [
                ...style$,
                this.itemHeight
                    ? {
                        height: this.itemHeight,
                    }
                    : {},
            ];
        };
        this.clearScrollTimer = () => {
            if (this.scrollTimer) {
                clearTimeout(this.scrollTimer);
                this.scrollTimer = null;
            }
        };
        this.clearMomentumTimer = () => {
            if (this.momentumTimer) {
                clearTimeout(this.momentumTimer);
                this.momentumTimer = null;
            }
        };
        this.initContainer = ({ nativeEvent }) => {
            this.containerHeight = nativeEvent.layout.height;
            this.initReady();
        };
        this.initItem = ({ nativeEvent }, index) => {
            this.itemHeightList[index] = nativeEvent.layout.height;
            this.initReady();
        };
        this.initReady = () => {
            if (this.initCount === this.props.children.length) {
                this.itemHeight = Math.max(...this.itemHeightList);
                this.paddingVertical = Math.floor((this.containerHeight - this.itemHeight) / 2);
                this.containerHeight = (this.paddingVertical * 2) + this.itemHeight;
                this.forceUpdate();
            }
            this.initCount += 1;
        };
        this.renderItems = () => {
            return this.props.children.map((item, index) => {
                const key = index;
                const initItem = event => this.initItem(event, index);
                return (React.createElement(item.type, Object.assign({}, item.props, { key: key, onLayout: initItem, style: this.getItemStyle(item.props.style) })));
            });
        };
        this.renderMask = () => {
            const mask = this.props.mask;
            return mask === true
                ? ['up', 'down'].map((item, index) => {
                    const key = index;
                    return (React.createElement(View, { key: item && key, style: { height: this.paddingVertical, backgroundColor: 'rgba(246,246,246,.8)' } }));
                })
                : mask;
        };
    }
    render() {
        const { initContainer } = this;
        const { mask, style } = this.props;
        if (!mask) {
            return (React.createElement(ScrollView, { onLayout: initContainer, showsVerticalScrollIndicator: false, style: [style, this.getContainerStyle()], ref: this.onScrollReady, scrollEventThrottle: 16, onScroll: this.onScroll, onScrollBeginDrag: this.onScrollBeginDrag, onScrollEndDrag: this.onScrollEndDrag, onMomentumScrollBegin: this.onMomentumScrollBegin, onMomentumScrollEnd: this.onMomentumScrollEnd },
                React.createElement(Select, { style: this.getSelectStyle() }, this.renderItems())));
        }
        return (React.createElement(View, { style: style },
            React.createElement(ScrollView, { onLayout: initContainer, showsVerticalScrollIndicator: false, style: this.getContainerStyle(), ref: this.onFakeReady },
                React.createElement(Select, { style: this.getSelectStyle() }, this.renderItems())),
            React.createElement(View, { style: styles.mask }, this.renderMask()),
            React.createElement(ScrollView, { style: [this.getContainerStyle(), styles.scroll], ref: this.onScrollReady, scrollEventThrottle: 16, onScroll: this.onScroll, onScrollBeginDrag: this.onScrollBeginDrag, onScrollEndDrag: this.onScrollEndDrag, onMomentumScrollBegin: this.onMomentumScrollBegin, onMomentumScrollEnd: this.onMomentumScrollEnd },
                React.createElement(Select, { style: this.getSelectStyle() }, this.renderItems()))));
    }
}
Picker.defaultProps = new Props();
//# sourceMappingURL=picker.js.map