import React, { Component, } from 'react';
import { Dimensions, View, } from 'react-native';
import DefaultTabBar from '../TabBar';
import ScrollView from '../ScrollView';
import Scene from './Scene';
import styles from './style';
const NOOP = () => { };
const DEVICE_WIDTH = Dimensions.get('window').width;
export var PositionEnum;
(function (PositionEnum) {
    PositionEnum["top"] = "top";
    PositionEnum["bottom"] = "bottom";
})(PositionEnum || (PositionEnum = {}));
export class TabSenceDefaultProps {
    constructor(defaultTextStyle, activeTextStyle) {
        this.children = [];
        this.initialPage = 0;
        this.onChangeTab = NOOP;
        this.page = 0;
        this.renderTabBar = null;
        this.scrollEnabled = false;
        this.style = null;
        this.tabBarPosition = PositionEnum.top;
        this.tabBarStyle = null;
        this.tabUnderLineStyle = null;
        this.defaultTextStyle = defaultTextStyle;
        this.activeTextStyle = activeTextStyle;
    }
}
export default class TabScene extends Component {
    constructor(props) {
        super(props);
        this.ComponentWillReceiveProps = (props) => {
            if (props.page >= 0 && props.page !== this.state.currentPage) {
                this.goToScene(props.page);
            }
        };
        this.getChildren = (children = this.props.children) => {
            return React.Children.map(children, child => child);
        };
        this.goToScene = (pageNumber) => {
            const currentPage = this.state.currentPage;
            this.updateSceneKeys({
                page: pageNumber,
                callback: this.handleChangeTab.bind(this, currentPage, pageNumber),
            });
            if (typeof this.onActive[pageNumber] === 'function') {
                this.onActive[pageNumber]();
            }
            if (typeof this.onDeactive[currentPage] === 'function') {
                this.onDeactive[currentPage]();
            }
        };
        this.updateSceneKeys = ({ page, callback = () => { } }) => {
            this.setState({ currentPage: page }, callback);
        };
        this.handleChangeTab = (prevPage, currentPage) => {
            this.props.onChangeTab({
                i: currentPage,
                ref: this.props.children[currentPage],
                from: prevPage,
            });
        };
        this.renderScrollableContent = () => {
            const contents = this.props.children.map(((child, index) => {
                const key = index;
                child.props.onActive && this.bindEvent(this.onActive, key, child.props.onActive);
                child.props.onDeactive && this.bindEvent(this.onDeactive, key, child.props.onDeactive);
                return (React.createElement(Scene, { key: child.key, style: child.props.style }, child));
            }));
            return contents;
        };
        this.renderTabBar = (props) => {
            if (typeof (this.props.renderTabBar) === 'function') {
                return null;
            }
            else if (this.props.renderTabBar) {
                return React.cloneElement(this.props.renderTabBar(props), props);
            }
            return React.createElement(DefaultTabBar, Object.assign({ ref: (ref) => { this.tabs = ref; }, style: { height: 30 } }, props));
        };
        this.renderScrollView = (isAnimated) => {
            if (isAnimated) {
                return (React.createElement(ScrollView, { currentPage: this.state.currentPage }, this.renderScrollableContent()));
            }
            return (React.createElement(ScrollView.Animate, { currentPage: this.state.currentPage }, this.renderScrollableContent()));
        };
        this.onActive = [];
        this.onDeactive = [];
        this.state = {
            currentPage: props.initialPage,
            containerWidth: DEVICE_WIDTH,
        };
    }
    get tabBarProps() {
        return {
            goToScene: this.goToScene,
            tabs: this.props.children.map(child => child.props.tabLabel),
            activeTab: this.state.currentPage,
            activeTextStyle: this.props.activeTextStyle,
            defaultTextStyle: this.props.defaultTextStyle,
            tabUnderLineStyle: this.props.tabUnderLineStyle,
            tabBarStyle: this.props.tabBarStyle,
        };
    }
    bindOnActive(key, callback) {
        this.onActive[key] = callback;
    }
    bindEvent(obj, key, callback) {
        const arr = obj;
        arr[key] = callback;
    }
    render() {
        return (React.createElement(View, { style: styles.container },
            this.props.tabBarPosition === 'top' && this.renderTabBar(this.tabBarProps),
            React.createElement(View, { style: { flex: 1 } }, this.props.scrollEnabled ?
                React.createElement(ScrollView, { currentPage: this.state.currentPage }, this.renderScrollableContent()) :
                React.createElement(ScrollView.Animate, { currentPage: this.state.currentPage }, this.renderScrollableContent())),
            this.props.tabBarPosition === 'bottom' && this.renderTabBar(this.tabBarProps)));
    }
}
TabScene.defaultProps = new TabSenceDefaultProps(styles.defaultTextStyle, styles.activeTextStyle);
//# sourceMappingURL=index.js.map